# International Personal Documentation Validator

Validates national identification documents across multiple countries using a 
pluggable service-based architecture.  
This module is intended for backend use and does not provide a UI.

---

## 🧩 What is this?

**IPD Validator** is a Drupal 10+ module that provides a generic API to 
validate personal identification documents (such as DNI, CUIT, or CUIL) 
by country. Validators are implemented as plugins. The system is extensible 
and allows new validators to be created without modifying the core module.

Example use cases:
- Validating national IDs in user registration or profile forms.
- Server-side data validation for integrations with country-specific document
formats.

---

## ⚙️ Features

- ✅ Provide document validation based on country rules.
- 🔌 Drop-in support for additional countries via plugins.
- 🧪 Comes with unit and kernel tests.
- 🚫 No UI: designed as backend validation service.

---

## 📦 Installation

1. Use composer to install the module code:
`composer require drupal/ipd_validator`.
2. Activate the module from the admin interface or using Drush:

```bash
drush en ipd_validator
```

## 🛠️ How to use

Use the `plugin.manager.ipd_validator` service to validate or format documents
from anywhere in your code.

### Validate a document

```php
$validatorManager = \Drupal::service('plugin.manager.ipd_validator');

// Check if an Argentine document is valid.
$isValid = $validatorManager->validate('20304050607', 'AR');
```

### Format a document

```php
$validatorManager = \Drupal::service('plugin.manager.ipd_validator');

// Try to normalize or format the document.
$formatted = $validatorManager->format('20304050607', 'AR');
```

### List supported countries

```php
$supported = $validatorManager->getSupportedCountries();
// Example: ['AR', 'ES', ...]
```

## 🧱 Plugin structure

Validators are implemented as annotated plugins, located in 
`src/Plugin/Validator`. They must all extend ValidatorPluginBase and implement
the `ValidatorInterface` interface.

### Basic plugin example

```php
/**
 * @Validator(
 *   id = "ar_dni_cuit_cuil_validator",
 *   label = @Translation("DNI, CUIT, or CUIL"),
 *   description = @Translation("Between 7 to 9 or 11 digits."),
 *   country_code = "AR",
 *   active = true
 * )
 */
final class ArDniCuitCuilValidator extends ValidatorPluginBase {
  public function isValid(string $document): bool {
    return preg_match('/^\d{7,9}$|^\d{11}$/', $document) === 1;
  }
}
```

> You can create your own module and add additional validators easily by
following this pattern.

## 📁 File structure (summary)

- src/Annotation/Validator.php – Defines the plugin annotation.
- src/ValidatorInterface.php – Contract that validators must implement.
- src/ValidatorPluginBase.php – Common base class with dependency injection.
- src/ValidatorPluginManager.php – Service that manages plugins and exposes the
main API.
- ipd_validator.services.yml – Declares the plugin.manager.ipd_validator
service.

## 🧪 Tests

This module includes automated tests:

- Unit tests for validator-specific logic.
- Kernel tests to verify the functionality of the entire plugin system.

You can run these tests with PHPUnit, in the standard Drupal 10 style.

## 🚧 Limitations

- If no validator exists for the requested country, the document is assumed to
be valid by default.
- The format() method simply returns the unchanged document, unless overridden
by the plugin.

## 👤 Maintenance

Module created and maintained by Pedro Peláez

Drupal.org nick: [@PSF_](https://www.drupal.org/u/psf_)

## 📘 Requirements

- Drupal core >= 10
- PHP >= 8.1

## 🗂️ Licensing

This module is distributed under the GPL-2.0 license (like all Drupal modules).
