<?php

declare(strict_types=1);

namespace Drupal\ipd_validator\Plugin\Validator;

use Drupal\ipd_validator\ValidatorPluginBase;

/**
 * Plugin implementation of the CPF/CNPJ validator for Brazil.
 *
 * @Validator(
 *   id = "br_cpf_cnpj_validator",
 *   label = @Translation("Brazil CPF or CNPJ"),
 *   description = @Translation("Must be between 11 and 14 digits. Full CPF validation."),
 *   country_code = "BR",
 *    active = true
 * )
 */
final class BrCpfCnpjValidator extends ValidatorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function isValid(string $document): bool {
    // Ensure the document contains only digits.
    if (!preg_match('/^\d{11,14}$/', $document)) {
      return FALSE;
    }

    $length = strlen($document);

    // Validate CPF (11 digits).
    if ($length === 11) {
      return $this->isValidCpf($document);
    }

    // Validate CNPJ (14 digits).
    if ($length === 14) {
      return $this->isValidCnpj($document);
    }

    return FALSE;
  }

  /**
   * Validates a Brazilian CPF using its check digits.
   *
   * @param string $cpf
   *   The CPF number.
   *
   * @return bool
   *   TRUE if the CPF is valid, FALSE otherwise.
   */
  private function isValidCpf(string $cpf): bool {
    // CPF cannot be all repeated numbers (e.g., "11111111111").
    if (preg_match('/^(\d)\1{10}$/', $cpf)) {
      return FALSE;
    }

    // Calculate the first check digit.
    $sum = 0;
    for ($i = 0; $i < 9; $i++) {
      $sum += (int) $cpf[$i] * (10 - $i);
    }
    $firstCheckDigit = ($sum % 11) < 2 ? 0 : 11 - ($sum % 11);

    // Calculate the second check digit.
    $sum = 0;
    for ($i = 0; $i < 10; $i++) {
      $sum += (int) $cpf[$i] * (11 - $i);
    }
    $secondCheckDigit = ($sum % 11) < 2 ? 0 : 11 - ($sum % 11);

    // Validate CPF check digits.
    return $cpf[9] == $firstCheckDigit && $cpf[10] == $secondCheckDigit;
  }

  /**
   * Validates a Brazilian CNPJ using its check digits.
   *
   * @param string $cnpj
   *   The CNPJ number.
   *
   * @return bool
   *   TRUE if the CNPJ is valid, FALSE otherwise.
   */
  private function isValidCnpj(string $cnpj): bool {
    // CNPJ cannot be all repeated numbers.
    if (preg_match('/^(\d)\1{13}$/', $cnpj)) {
      return FALSE;
    }

    // Weight multipliers for CNPJ validation.
    $weights1 = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
    $weights2 = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];

    // First check digit.
    $sum = 0;
    for ($i = 0; $i < 12; $i++) {
      $sum += (int) $cnpj[$i] * $weights1[$i];
    }
    $firstCheckDigit = ($sum % 11) < 2 ? 0 : 11 - ($sum % 11);

    // Second check digit.
    $sum = 0;
    for ($i = 0; $i < 13; $i++) {
      $sum += (int) $cnpj[$i] * $weights2[$i];
    }
    $secondCheckDigit = ($sum % 11) < 2 ? 0 : 11 - ($sum % 11);

    // Validate CNPJ check digits.
    return $cnpj[12] == $firstCheckDigit && $cnpj[13] == $secondCheckDigit;
  }

}
