<?php

declare(strict_types=1);

namespace Drupal\ipd_validator\Plugin\Validator;

use Drupal\ipd_validator\ValidatorPluginBase;

/**
 * Plugin implementation of the Chile CI/RUT validator.
 *
 * @Validator(
 *   id = "cl_ci_rut_validator",
 *   label = @Translation("Chile CI or RUT"),
 *   description = @Translation("Must be a valid Chilean RUT (with correct check digit)."),
 *   country_code = "CL",
 *    active = true
 * )
 */
final class ClCiRutValidator extends ValidatorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function isValid(string $document): bool {
    // Remove dots, hyphens, and convert to lowercase.
    $document = strtolower(str_replace(['.', '-'], '', $document));

    // Length should be at least 2 characters (1 digit + 1 verifier)
    if (strlen($document) < 2) {
      return FALSE;
    }

    // Extract body and check digit.
    $body = substr($document, 0, -1);
    $verifier = substr($document, -1);

    if (!ctype_digit($body)) {
      return FALSE;
    }

    return $this->verifyRut($body, $verifier);
  }

  /**
   * Verifies the RUT check digit using modulo 11.
   *
   * @param string $body
   *   The numeric part of the RUT.
   * @param string $verifier
   *   The expected check digit ('0'-'9' or 'K').
   *
   * @return bool
   *   TRUE if the check digit is correct.
   */
  private function verifyRut(string $body, string $verifier): bool {
    $reverseDigits = array_reverse(str_split($body));
    $sum = 0;
    $multiplier = 2;

    foreach ($reverseDigits as $digit) {
      $sum += (int) $digit * $multiplier;
      $multiplier = $multiplier < 7 ? $multiplier + 1 : 2;
    }

    $modulo = 11 - ($sum % 11);
    $expected = '';

    if ($modulo === 11) {
      $expected = '0';
    }
    elseif ($modulo === 10) {
      $expected = 'k';
    }
    else {
      $expected = (string) $modulo;
    }

    return $verifier === $expected;
  }

}
