<?php

declare(strict_types=1);

namespace Drupal\ipd_validator\Plugin\Validator;

use Drupal\ipd_validator\ValidatorPluginBase;

/**
 * Plugin implementation of the China Citizen ID validator.
 *
 * @Validator(
 *   id = "cn_id_validator",
 *   label = @Translation("China Citizen ID Number"),
 *   description = @Translation("Must be 18 characters: 17 digits followed by 1 digit or 'X'."),
 *   country_code = "CN",
 *    active = true
 * )
 */
final class CnIdValidator extends ValidatorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function isValid(string $document): bool {
    // Uppercase the last character in case of 'x'.
    $document = strtoupper($document);

    // Validate basic format: 17 digits + 1 digit or 'X'.
    if (!preg_match('/^\d{17}[\dX]$/', $document)) {
      return FALSE;
    }

    return $this->isValidChecksum($document);
  }

  /**
   * Validates the checksum digit of the Chinese ID.
   *
   * @param string $document
   *   The 18-character citizen ID.
   *
   * @return bool
   *   TRUE if checksum is valid.
   */
  private function isValidChecksum(string $document): bool {
    // Coefficients for checksum calculation.
    $weights = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2];
    $checkDigits = ['1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2'];

    $sum = 0;

    for ($i = 0; $i < 17; $i++) {
      $sum += (int) $document[$i] * $weights[$i];
    }

    $mod = $sum % 11;
    $expected = $checkDigits[$mod];

    return $document[17] === $expected;
  }

}
