<?php

declare(strict_types=1);

namespace Drupal\ipd_validator\Plugin\Validator;

use Drupal\ipd_validator\ValidatorPluginBase;

/**
 * Plugin implementation of the Thailand ID validator.
 *
 * @Validator(
 *   id = "th_id_validator",
 *   label = @Translation("Thailand Identity Card"),
 *   description = @Translation("Must be exactly 13 numeric digits."),
 *   country_code = "TH",
 *   active = true
 * )
 */
final class ThIdValidator extends ValidatorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function isValid(string $document): bool {
    // Must be exactly 13 digits.
    if (!preg_match('/^\d{13}$/', $document)) {
      return FALSE;
    }

    return $this->isValidThaiId($document);
  }

  /**
   * Validate Thai ID using Mod 11 check digit.
   *
   * @param string $id
   *   The full 13-digit ID.
   *
   * @return bool
   *   TRUE if check digit is valid.
   */
  private function isValidThaiId(string $id): bool {
    $sum = 0;
    for ($i = 0; $i < 12; $i++) {
      $sum += (int) $id[$i] * (13 - $i);
    }

    $checkDigit = (11 - ($sum % 11)) % 10;

    return (int) $id[12] === $checkDigit;
  }

}
