<?php

declare(strict_types=1);

namespace Drupal\ipd_validator\Plugin\Validator;

use Drupal\ipd_validator\ValidatorPluginBase;

/**
 * Plugin implementation of the South Africa ID validator.
 *
 * @Validator(
 *   id = "za_id_validator",
 *   label = @Translation("South Africa Identity Card"),
 *   description = @Translation("Must be exactly 13 digits with valid Luhn check."),
 *   country_code = "ZA",
 *   active = false
 * )
 */
final class ZaIdValidator extends ValidatorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function isValid(string $document): bool {
    if (!preg_match('/^\d{13}$/', $document)) {
      return FALSE;
    }

    return $this->luhnCheck($document);
  }

  /**
   * Apply the Luhn algorithm to validate the last digit.
   *
   * @param string $number
   *   The full 13-digit SA ID number.
   *
   * @return bool
   *   TRUE if valid, FALSE otherwise.
   */
  private function luhnCheck(string $number): bool {
    $sum = 0;
    $alt = FALSE;

    // Start from the rightmost digit.
    for ($i = strlen($number) - 1; $i >= 0; $i--) {
      $n = (int) $number[$i];
      if ($alt) {
        $n *= 2;
        if ($n > 9) {
          $n -= 9;
        }
      }
      $sum += $n;
      $alt = !$alt;
    }

    return ($sum % 10) === 0;
  }

}
