<?php

declare(strict_types=1);

namespace Drupal\ipd_validator;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\ipd_validator\Annotation\Validator;

/**
 * Validator plugin manager.
 */
class ValidatorPluginManager extends DefaultPluginManager {

  /**
   * Constructs the object.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/Validator', $namespaces, $module_handler, ValidatorInterface::class, Validator::class);
    $this->alterInfo('validator_info');
    $this->setCacheBackend($cache_backend, 'validator_plugins');
  }

  /**
   * Validate a Personal Identification Document.
   *
   * If no validator exists for the country, the document is considered valid
   * by default.
   *
   * @param string $document
   *   The document code.
   * @param string $countryCode
   *   The document country (ISO 3166-1 alpha-2).
   *
   * @return bool
   *   TRUE if the document is valid according to the country validator,
   *   or if none exists.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function validate(string $document, string $countryCode): bool {
    $definitions = $this->getDefinitions();
    $found = FALSE;

    foreach ($definitions as $plugin_id => $definition) {
      if (
        strcasecmp($definition['country_code'], $countryCode) === 0 &&
        (!isset($definition['active']) || $definition['active'] === TRUE)
      ) {
        $validator = $this->createInstance($plugin_id);

        if ($validator->isActive()) {
          $found = TRUE;
          if ($validator->isValid($document)) {
            return TRUE;
          }
        }
      }
    }

    // If no validator was found for the country, assume it's valid by default.
    return !$found;
  }

  /**
   * Format a Personal Identification Document.
   *
   * If no formatter plugin exists for the country, returns the original
   * document.
   *
   * @param string $document
   *   The document code.
   * @param string $countryCode
   *   The document country (ISO 3166-1 alpha-2).
   *
   * @return string
   *   The formatted document, or original if no formatter found.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function format(string $document, string $countryCode): string {
    $definitions = $this->getDefinitions();

    foreach ($definitions as $plugin_id => $definition) {
      if (
        strcasecmp($definition['country_code'], $countryCode) === 0 &&
        (!isset($definition['active']) || $definition['active'] === TRUE)
      ) {
        $validator = $this->createInstance($plugin_id);
        if ($validator->isActive()) {
          return $validator->format($document);
        }
      }
    }

    // No formatter found for target country, return original.
    return $document;
  }

  /**
   * Get list of country codes with active validators.
   *
   * @return string[]
   *   An array of ISO country codes.
   */
  public function getSupportedCountries(): array {
    $definitions = $this->getDefinitions();
    $countries = [];

    foreach ($definitions as $definition) {
      if (
        (!isset($definition['active']) || $definition['active'] === TRUE)
      ) {
        $code = strtoupper($definition['country_code']);
        $countries[$code] = $code;
      }
    }

    // Return unique country codes.
    return array_values($countries);
  }

}
