<?php

declare(strict_types=1);

namespace Drupal\Tests\ipd_validator\Kernel;

use Drupal\ipd_validator\ValidatorInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\ipd_validator\ValidatorPluginManager;

/**
 * Tests for the ValidatorPluginManager functionality.
 *
 * @group ipd_validator
 */
final class ValidatorPluginManagerTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'ipd_validator',
  ];

  /**
   * The plugin manager under test.
   *
   * @var \Drupal\ipd_validator\ValidatorPluginManager
   */
  protected ValidatorPluginManager $pluginManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->pluginManager = $this->container->get('plugin.manager.ipd_validator');
  }

  /**
   * Tests the validate() method.
   */
  public function testValidate(): void {
    // Should be valid for CUIT (11 digits).
    $this->assertTrue($this->pluginManager->validate('20304050607', 'AR'));

    // Should be valid for DNI (7-9 digits).
    $this->assertTrue($this->pluginManager->validate('12345678', 'AR'));

    // Should be invalid (6 digits not accepted).
    $this->assertFalse($this->pluginManager->validate('123456', 'AR'));

    // Non-existent country should return TRUE (accepted by default).
    $this->assertTrue($this->pluginManager->validate('123456', 'ZZ'));
  }

  /**
   * Tests the format() method.
   */
  public function testFormat(): void {
    // Default ArDniCuitCuilValidator does not override format(),
    // so result is same.
    $input = '20304050607';
    $this->assertEquals($input, $this->pluginManager->format($input, 'AR'));

    // Non-existent plugin: should return unchanged input.
    $this->assertEquals('ABC123XYZ', $this->pluginManager->format('ABC123XYZ', 'ZZ'));
  }

  /**
   * Tests getSupportedCountries() method.
   */
  public function testGetSupportedCountries(): void {
    $countries = $this->pluginManager->getSupportedCountries();

    // Check that 'AR' (Argentina) appears in the supported list.
    $this->assertContains('AR', $countries);

    // Check that codes are in uppercase ISO format.
    foreach ($countries as $code) {
      $this->assertMatchesRegularExpression('/^[A-Z]{2}$/', $code);
    }
  }

  /**
   * Verifies that all registered validator plugins can be instantiated.
   */
  public function testAllPluginsAreInstantiable(): void {
    $definitions = $this->pluginManager->getDefinitions();

    $this->assertNotEmpty($definitions, 'At least one plugin definition should exist.');

    foreach ($definitions as $plugin_id => $definition) {
      try {
        $instance = $this->pluginManager->createInstance($plugin_id);
        $this->assertInstanceOf(ValidatorInterface::class, $instance, "Plugin $plugin_id is instantiable.");
      }
      catch (\Throwable $e) {
        $this->fail("Plugin '$plugin_id' could not be instantiated: " . $e->getMessage());
      }
    }
  }

}
