<?php

declare(strict_types=1);

namespace Drupal\Tests\ipd_validator\Unit\Plugin\Validator;

use Drupal\Core\Locale\CountryManager;
use Drupal\ipd_validator\Plugin\Validator\BdNidValidator;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\ipd_validator\Plugin\Validator\BdNidValidator
 *
 * @group ipd_validator
 */
final class BdNidValidatorTest extends UnitTestCase {

  /**
   * The CountryManager mock.
   *
   * @var \Drupal\Core\Locale\CountryManager|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $countryManager;

  /**
   * The BdNidValidator instance.
   *
   * @var \Drupal\ipd_validator\Plugin\Validator\BdNidValidator
   */
  protected BdNidValidator $validator;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Mock CountryManager (required for ValidatorPluginBase).
    $this->countryManager = $this->createMock(CountryManager::class);

    // Create an instance of the validator.
    $this->validator = new BdNidValidator(
      [],
      'bd_nid_validator',
      ['country_code' => 'BD'],
      $this->countryManager
    );
  }

  /**
   * Test the document validation using a data provider.
   *
   * @param string $document
   *   The document number to be validated.
   * @param bool $expected
   *   The expected validation result.
   *
   * @covers ::isValid
   * @dataProvider documentProvider
   */
  public function testDocumentValidation(string $document, bool $expected): void {
    $this->assertSame($expected, $this->validator->isValid($document));
  }

  /**
   * Provides a dataset of valid and invalid Bangladesh NID numbers.
   *
   * @return array
   *   A list of [document, expected_result] pairs.
   */
  public function documentProvider(): array {
    return [
      // ✅ Valid NIDs (13 to 17 digits)
    // 13 digits
      ['1234567890123', TRUE],
    // 15 digits
      ['123456789012345', TRUE],
    // 17 digits
      ['12345678901234567', TRUE],

      // ❌ Invalid cases
    // 12 digits (too short)
      ['123456789012', FALSE],
    // 18 digits (too long)
      ['123456789012345678', FALSE],
    // Contains letters.
      ['ABC1234567890123', FALSE],
    // Mixed alphanumeric.
      ['1234A56789012345', FALSE],
    // Too short (only 8 digits)
      ['12345678', FALSE],
    // Empty string.
      ['', FALSE],
    // Completely non-numeric.
      ['abcdefghijklmno', FALSE],
    ];
  }

}
